/// <reference path="../../../node_modules/@types/jquery/JQueryStatic.d.ts" />
import { __awaiter, __rest } from "tslib";
import { BaseHttpClient, } from "@odata2ts/http-client-base";
import { getDefaultConfig, mergeAjaxConfig } from "./AjaxRequestConfig.js";
import { JQueryClientError } from "./JQueryClientError.js";
export const DEFAULT_ERROR_MESSAGE = "No error message!";
export class JQueryClient extends BaseHttpClient {
    constructor(jquery, config, clientOptions) {
        super(clientOptions);
        this.client = jquery;
        this.config = getDefaultConfig(config);
    }
    mapHeaders(jqXhr) {
        return jqXhr
            .getAllResponseHeaders()
            .trim()
            .split(/[\r\n]+/)
            .reduce((collector, line) => {
            const parts = line.split(": ");
            const header = parts.shift();
            const value = parts.join(": ");
            if (header) {
                collector[header.toLowerCase()] = value;
            }
            return collector;
        }, {});
    }
    executeRequest(method, url, data, requestConfig, internalConfig = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const withInternalConfig = mergeAjaxConfig({ headers: internalConfig.headers }, requestConfig);
            const _a = mergeAjaxConfig(this.config, withInternalConfig), { params } = _a, mergedConfig = __rest(_a, ["params"]);
            mergedConfig.method = method;
            mergedConfig.data = JSON.stringify(data);
            mergedConfig.url = url;
            if (params && Object.values(params).length) {
                mergedConfig.url +=
                    (url.match(/\?/) ? "&" : "?") +
                        // @ts-ignore
                        new URLSearchParams(params).toString();
            }
            if (internalConfig.dataType === "blob") {
                mergedConfig.xhrFields = { responseType: "blob" };
            }
            else if (internalConfig.dataType === "stream") {
                throw new Error("Streaming is not supported by the JqueryClient!");
            }
            // the actual request
            return new Promise((resolve, reject) => {
                this.client.ajax(Object.assign(Object.assign({}, mergedConfig), { success: (response, textStatus, jqXHR) => {
                        resolve({
                            status: jqXHR.status,
                            statusText: jqXHR.statusText,
                            headers: this.mapHeaders(jqXHR),
                            data: response,
                        });
                    }, error: (jqXHR, textStatus, thrownError) => {
                        const responseMessage = this.retrieveErrorMessage(jqXHR === null || jqXHR === void 0 ? void 0 : jqXHR.responseJSON);
                        const failMsg = responseMessage || thrownError || DEFAULT_ERROR_MESSAGE;
                        const errorMessage = responseMessage ? "OData server responded with error: " + responseMessage : failMsg;
                        const responseHeaders = this.mapHeaders(jqXHR);
                        reject(new JQueryClientError(errorMessage, jqXHR.status, responseHeaders, new Error(failMsg), jqXHR));
                    } }));
            });
        });
    }
}
//# sourceMappingURL=JQueryClient.js.map